#include <mex.h>

#include "ransac.h"

void mexFunction(int aDstN, mxArray **aDstP, int aSrcN, const mxArray **aSrcP) {
	if (aSrcN >= 2) {
		long max_sam = MAX_SAMPLES;
		double confidence = CONFIDENCE;
		double *u = mxGetPr(aSrcP[0]);
		int len = mxGetN(aSrcP[0]);
		double *th = mxGetPr(aSrcP[1]);
		int do_lo = 1;
		double *F;
		mwSize dims[2];
		unsigned char *inliers = 0;
		unsigned stats[3];
		double *statsD = 0;
		unsigned inlLimit = INL_LIMIT_F;
		unsigned i;

		/* check u matrix dimensions */
		if (!mxIsDouble(aSrcP[0]) || mxGetM(aSrcP[0]) != 6) {
			mexErrMsgTxt("DOUBLE 6xN matrix expected as first parameter (point pairs in homogenous coordinates).\n\n");
			return;
		}
		if (mxGetN(aSrcP[0]) < 7) {
			mexErrMsgTxt("At least 7 correspondences required.\n\n");
			return;
		}

		/* check th */
		if (!mxIsDouble(aSrcP[1]) || mxGetNumberOfElements(aSrcP[1]) != 1) {
			mexErrMsgTxt("DOUBLE scalar expected as second parameter (threshold).\n\n");
			return;
		}

		/* check iter_type */
		if (aSrcN > 2) {
			if (mxGetNumberOfElements(aSrcP[2]) != 1) {
				mexErrMsgTxt("Scalar value expected as third parameter (LO on/off).\n\n");
				return;
			}
			do_lo = (int)(mxGetScalar(aSrcP[2])>0);
		}

		/* check conf */
		if (aSrcN > 3) {
			confidence = (double)(mxGetScalar(aSrcP[3]));
			if (confidence > 1) {
				mexErrMsgTxt("DOUBLE scalar expected as fourth parameter (confidence) and should be less then or equal to 1.\n\n");
				return;
			}
		}

		/* inliers limit */
		if (aSrcN>4) {
			double il = mxGetScalar(aSrcP[4]);
			if (!mxIsInf(il) && il >= 1) {
				inlLimit = (unsigned)il;
			} else {
				inlLimit = 0;
			}
		}

		/* maximal sample count */
		if (aSrcN > 5) {
			if (mxGetScalar(aSrcP[5]) > LONG_MAX) {
				mexWarnMsgTxt("Maximum number of samples set too high! Limited to a range of UNSIGNED LONG! Using default value instead.\n\n");
			} else {
				max_sam = (unsigned long)mxGetScalar(aSrcP[5]);
			}
		}

		/* random seed */
		if (aSrcN > 6) {
			srand((int)mxGetScalar(aSrcP[6]));
		}

		/* OUT: H */
		aDstP[0] = mxCreateDoubleMatrix(3, 3, mxREAL);
		F = mxGetPr(aDstP[0]);

		/* OUT: inliers */
		if (aDstN > 1) {
			dims[0] = 1; dims[1] = len;
			aDstP[1] = mxCreateLogicalArray(2, dims);
			inliers = (unsigned char *) mxGetData(aDstP[1]);
		}

		/* THE MAIN FUNCTION CALL */
		ransacF(u, len, *th, confidence, max_sam, do_lo, inlLimit, F, inliers, stats);

		/* OUT: statistics */
		if (aDstN > 2) {
			aDstP[2] = mxCreateDoubleMatrix(1, 3, mxREAL);
			statsD = mxGetPr(aDstP[2]);
			for (i = 0; i < 3; ++i) { /* convert to double for Matlab */
				statsD[i] = (double) stats[i];
			}
		}

	} else {
		mexPrintf("Usage:\n\n");
		mexPrintf("  [F, INL, STATS] = loransacF(TC_PAIRS, THRESHOLD)\n");
		mexPrintf("  [F, INL, STATS] = loransacF(TC_PAIRS, THRESHOLD[, LO, CONF, INL_LIMIT, MAX_SAMPLES, RAND_SEED])\n\n"); 
		mexPrintf("Robustly estimates fundamental matrix from a set of TC_PAIRS in\n");
		mexPrintf("homogenous coordinates. Inliers to the epipolar geometry are established\n");
		mexPrintf("by comparing their distance from the model against THRESHOLD. Optionally\n");
		mexPrintf("a Local Optimisation is employed (LO > 0) or not (LO == 0).\n");
		mexPrintf("Maximum number of samples in the RANSAC iteration is limited to \n");
		mexPrintf("MAX_SAMPLES (default = %d). Number of inliers for the computation of\n", MAX_SAMPLES);
		mexPrintf("least squares is limited to INL_LIMIT (set to 0 to turn this limit off).\n");
		mexPrintf("Optional outputs are INL - logical array indicating inliers to the\n");
		mexPrintf("resulting homography and STATS - statistics of the run in the format:\n");
		mexPrintf("  [#samples drawn, #local optimisations performed, #models rejected].\n");
	}
}

